from pprint import pprint
import pandas as pd
import openpyxl
from openpyxl.styles.borders import Border, Side
from openpyxl.styles import PatternFill
from openpyxl.styles import Font
import datetime
from openpyxl.comments import Comment



def checkChild(data):
    """
    テーブルからnodeごとの子供リストを返す
    """

    dictOut = {}
    for index, row in enumerate(data):
        # print(f'----- {index} -----')
        # print(row)
        dict1 = data[index]
        for cnt in range(index + 1, len(data)):
            # print(cnt)
            dict2 = data[cnt]
            if dict2['LEVEL'] <= dict1['LEVEL']:
                break

            if dict1['LEVEL'] + 1 == dict2['LEVEL']:
                # print(dict2)
                if dict1["ID"] not in dictOut:
                    dictOut[dict1["ID"]] = [dict2["ID"]]
                else:
                    dictOut[dict1["ID"]].append(dict2["ID"])

    return dictOut


def checkTree(data_org, data_new):
    """
    テーブルからnodeごとの子供リストを返す
    """

    dictOrg = checkChild(data_org)
    dictNew = checkChild(data_new)
    dictDiff = {}

    for key in dictNew:
        if key in dictOrg:
            # dictDiff[key] = {"追加": dictNew[key]}
            list_tree_add = [item for item in dictNew[key]
                             if item not in dictOrg[key]]
            if len(list_tree_add) > 0:
                # print(f'add key:{key} {list_tree_add}')
                if key in dictDiff:
                    dictDiff[key]['追加'] = list_tree_add
                else:
                    dictDiff[key] = {'追加': list_tree_add}

            list_tree_del = [item for item in dictOrg[key]
                             if item not in dictNew[key]]
            if len(list_tree_del) > 0:
                # print(f'del key:{key} {list_tree_del}')
                if key in dictDiff:
                    dictDiff[key]['削除'] = list_tree_del
                else:
                    dictDiff[key] = {'削除': list_tree_del}

        else:
            # print(f'子供がなかったが子供ができた key:{key} {dictNew[key]}')
            if key in dictDiff:
                dictDiff[key]['子供がなかったが子供ができた'] = dictNew[key]
            else:
                dictDiff[key] = {'子供がなかったが子供ができた': dictNew[key]}

    for key in dictOrg:
        if key not in dictNew:
            # print(f'子供がなくなった key:{key} {dictOrg[key]}')
            dictDiff[key] = {'子供がなくなった': dictOrg[key]}

    return dictDiff


def find_dictionary_by_id(data, target_id):
    for dictionary in data:
        if dictionary["ID"] == target_id:
            return dictionary
    return None


def diffAttrTable(data_org, data_new):
    """
    data_rewに対して、data_orgとの差分を返す
    """

    dictOut = {}
    for row in data_new:
        new_id = row["ID"]
        dictOrg = find_dictionary_by_id(data_org, new_id)
        if dictOrg is None:
            dictOut[new_id] = {"行追加": row}
        else:
            dictOut[new_id] = diffAttrRow(dictOrg, row)

    for row in data_org:
        org_id = row["ID"]
        dictOrg = find_dictionary_by_id(data_new, org_id)
        if dictOrg is None:
            dictOut[org_id] = {"行削除": row}

    return dictOut


def diffAttrRow(data_org, data_new):
    """  
    二つの辞書の変更、追加、削除をチェックして結果を辞書で返す
    """

    dictDiff = {}
    dictDiff['変更'] = {}
    dictDiff['追加'] = {}
    dictDiff['削除'] = {}

    for key in data_org:
        val = data_org[key]
        if key not in data_new:
            dictDiff['削除'][key] = val

    for key in data_new:
        val = data_new[key]
        if key not in data_org:
            dictDiff['追加'][key] = val
        else:
            if val != data_org[key]:
                dictDiff['変更'][key] = {"旧": data_org[key], "新": val}

    return dictDiff


def excel2data(excel_file):
    """
    Excelファイルを読み込んでデータを返す
    """

    df = pd.read_excel(excel_file, sheet_name='Sheet1')
    # print(df)

    return df.to_dict('records')


if __name__ == "__main__":

    data_1 = excel2data('data_org.xlsx')
    data_2 = excel2data('data_new.xlsx')
    # pprint(ret)
    dictDiff = diffAttrTable(data_1, data_2)
    # pprint(dictDiff)

    tree_org = checkChild(data_1)
    tree_new = checkChild(data_2)
    # pprint(tree_org)
    # pprint(tree_new)
    # print('*****')
    diffTree = checkTree(data_1, data_2)
    # pprint('--------diff tree -----------')
    # pprint(diffTree)

    wb = openpyxl.load_workbook('data_new.xlsx')
    ws = wb['Sheet1']

    for row in range(1,  ws.max_row + 1):
        for col in range(1,  ws.max_column + 1):
            key = ws.cell(row=row, column=1).value
            attr = ws.cell(row=1, column=col).value
            data = ws.cell(row=row, column=col).value
            flag = ""
            if key in dictDiff:
                if '変更' in dictDiff[key]:
                    if attr in dictDiff[key]['変更']:
                        flag = "あり"
                        fill = PatternFill(
                            patternType='solid', fgColor='FFFFAA')
                        ws.cell(row=row, column=col).fill = fill
                        ws.cell(row=row, column=col).comment = Comment(
                            f"旧:{dictDiff[key]['変更'][attr]['旧']}", "system")
                if '行追加' in dictDiff[key]:
                    fill = PatternFill(patternType='solid', fgColor='FFAAFF')
                    ws.cell(row=row, column=col).fill = fill

            # print(f'{key} {attr} {data} {flag}')

        id=ws.cell(row=row, column=1).value
        if id in diffTree:
            ws.cell(row=row, column=1).comment = Comment(diffTree[id], "system")


    # datetimeを文字列へ
    today = datetime.datetime.now()
    str_day = today.strftime("%Y%m%d_%H%M%S")

    wb.save(f'out_{str_day}.xlsx')
    wb.close()

